;
$DATE (07.03.2006)
$TITLE (TmTest.a51)
$PAGELENGTH(56)
$PAGEWIDTH(150)
$DEBUG
$XREF
$NOLIST
$NOMOD51
$INCLUDE(89S8252.MCU)
$LIST
;
;********************************************************************************************************
; Temperaturanzeige mit Min-Max-Werten  auf LCD
; fr Temperatursensor LM 75
; J. Rathlev, Juni 2006
;********************************************************************************************************

      NAME      MMTherm

;********************************************************************************************************
; Konstanten
; ==========

CODEMEM EQU     8000H

; Berechnung des Zhlerwertes:
;     N = 65536 - T/s * f/MHz / 12  

; f(Takt) = 24 MHz: 10 ms ==> 45536

; f(Takt) = 22,1184 MHz: 10 ms ==> 47104
 
 
TIMCNT  EQU     47104   ; Zhlerwert fr 10 ms bei 22,1184 MHz

TIntvl  EQU     100      ; Anzahl der 10ms Takte fr 1000 ms
                
;********************************************************************************************************
; RAM-Daten
; =========

        DSEG    AT      30H

TmSAdr: DS      1               ; Subadresse des Portexpanders
TmDat:  DS      2   
TmMin:  DS      2
TmMax:  DS      2
TCount: DS      1

        BSEG      
        
TmFlag: DBIT    1               ; Flag fr neuen Temperaturwert

;********************************************************************************************************
; ext. Datenspeicher
; ==================

        XSEG    AT 0C000H

;********************************************************************************************************
; Programmstart

        CSEG    AT      CODEMEM

        LJMP    Start

        ORG     CODEMEM+ExtI0

;        LJMP    EX0IRQ
        
        ORG     CODEMEM+Timer0

        LJMP    T0IRQ       
        
;********************************************************************************************************

        ORG     CODEMEM+30H

$INCLUDE(i2c.a51)
$INCLUDE(Display-162.a51)
$INCLUDE(Display.a51)
$INCLUDE(Tast3x4.a51)

;********************************************************************************************************
; Timer 0 Interrupt

USING   2                       ; Registerbank 2 verwenden

T0IRQ:  PUSH    PSW
        MOV     PSW,#010H       ; Registerbank 2 einschalten

        MOV     TH0,#HIGH(TIMCNT)
        MOV     TL0,#LOW(TIMCNT)
        DJNZ    TCOUNT,T0RET

        MOV     R0,#TmDat
        MOV     R1,#2
        MOV     R2,TmSAdr
        CALL    TmRead
        
        SETB    TmFlag          ; neuer Wert
        MOV     TCOUNT,#TIntvl

T0RET:  POP     PSW             ; Flags wiederherstellen 
        RETI
   
USING   0                       ; Registerbank 0 verwenden


;********************************************************************************************************
; Initialisieren

InitTmSens:
        MOV     TmSAdr,A
        CLR     TmFlag
        RET        


;********************************************************************************************************
; Timer 0 initialisieren, Interrupts freischalten

InitTimer:
        MOV     A,TMOD      
        ANL     A,#0F0H         ; Ausmaskieren von Timer 0 (Bit 0..3)
        ORL     A,#01H          ; Setze Mode 1, int. Zhltakt
        MOV     TMOD,A
        MOV     TH0,#HIGH(TIMCNT) ; Anfangswert fr 10 ms
        MOV     TL0,#LOW(TIMCNT)
        CLR     TF0
        MOV     TCOUNT,#TIntvl  ; Zhler fr Zeitintervall
        SETB    TR0             ; Starte Timer
        SETB    ET0             ; Enable Timer-Interrupt
        RET

;********************************************************************************************************
; Werte vergleichen

Compare:
        INC     R0
        INC     R1
        MOV     A,@R0           ; low
        SUBB	  A,@R1
        DEC     R0
        DEC     R1
        MOV     A,@R1           ; high
        CPL     ACC.7  
        MOV     R2,A                                                                                 
        MOV     A,@R0           
        CPL     ACC.7
        SUBB	  A,R2  
        RET
        
;********************************************************************************************************
; Hauptprogramm

Start:  MOV     SP,#STACKSTART
        MOV     A,#0            ; Subadresse
        CALL    InitTmSens
        CALL    InitKey
        CALL    InitDisplay

        MOV     DPTR,#AuTab     ; zus. Zeichen definieren
        CALL    DefChar
        MOV     DPTR,#AdTab
        CALL    DefChar       
        
        CALL    LgDel
        MOV     A,#1100B        ; Display ein, Cursor aus, Cursor nicht blinken
        CALL    WDCtrl

        MOV     TmMin,#7FH
        MOV     TmMin+1,#0FFH   ; auf Max. setzen
        MOV     TmMax,#80H
        MOV     TmMax+1,#0      ; auf Min. setzen
        
        CALL    InitTimer
        SETB    EA              ; Enable Global-Interrupt

LOOP:   CALL    GetKey
        JNC     Lp8             ; keine Eingabe
        CJNE    A,#'*',Lp7
        MOV     TmMin,TmDat
        MOV     TmMin+1,TmDat+1
        SJMP    Lp5        
        
Lp7:    CJNE    A,#'#',Lp8
        MOV     TmMax,TmDat
        MOV     TmMax+1,TmDat+1
        SJMP    Lp5        

Lp8:    JNB     TmFlag,LOOP     ; warte
        CLR     TmFlag
        
        ; Min. setzen
        CLR     C
        MOV     R0,#TmDat
        MOV     R1,#TmMin
        CALL    Compare
        JNC     Lp6
        MOV     TmMin,TmDat
        MOV     TmMin+1,TmDat+1

        ; Max. setzen
Lp6:    SETB    C
        MOV     R0,#TmDat
        MOV     R1,#TmMax
        CALL    Compare
        JC      Lp5
        MOV     TmMax,TmDat
        MOV     TmMax+1,TmDat+1
                
Lp5:    MOV     A,#0
        CALL    SDLine
        MOV     DPTR,#Text1
        CALL    WDStr

        MOV     R0,#TmDat
        CALL    WDVal          ; Wert anzeigen

        MOV     A,#1
        CALL    SDLine
        MOV     A,#2
        CALL    WDChar
        MOV     R0,#TmMin
        CALL    WDVal          ; Min. anzeigen

        MOV     A,#1
        MOV     R2,#8
        CALL    SDLPos
        MOV     A,#1
        CALL    WDChar
        MOV     R0,#TmMax      ; Max. anzeigen
        CALL    WDVal
        SJMP    LOOP
        
; Wert auf Anzeige ausgeben
WdVal:  MOV     A,@R0
        INC     R0
        MOV     B,A
        JNB     ACC.7,Lp2     ; positiv
        CPL     A
        INC     A               ; 2er-Komplement
        MOV     B,A
        MOV     A,#'-'
        CALL    WDChar       ; Vorzeichen
        SJMP    Lp2
        
Lp1:    MOV     A,#32           
        CALL    WDChar       ; Leerzeichen
Lp2:    MOV     A,B
        CALL    WDDec 
        MOV     A,#','
        CALL    WDChar       ; Komma 
        MOV     A,@R0        ; Nachkommastelle
        JB      ACC.7,Lp3
        MOV     A,#'0'
        SJMP    Lp4
        
Lp3:    MOV     A,#'5'
Lp4:    CALL    WDChar       ; letzte Stelle
        
        MOV     DPTR,#TEXT3
        CALL    WDStr
        RET
                
Text1:  DB      'T = ',0
Text2:  DB      ' ',186,'C',0
Text3:  DB      0DFH,'C',0

;********************************************************************************************************
        DSEG

STACKSTART:
        DS      1       ; stack: rest of data memory

END
