$NOLIST

; Include Datei

;********************************************************************************************************
; Lesen und Schreiben von Bytes ber die serielle Schnittstelle (direkt)
; J. Rathlev, Dez. 2019

; Enthaltene Routinen:
;  InitSerial      - Initialisieren der Schnittstelle
;  WriteChar       - Ein Zeichen ausgeben
;  WriteString     - String aus dem erweiteren Arbeitsspeicher ausgeben
;  WriteCrLf       - Neue Zeile (Cr+Lf) ausgeben
;  WriteFixString  - String aus dem Programmspeicher ausgeben
;  WriteDecByte    - Ein Byte als Dezimalzahl ausgeben
;  WriteDec        - 16-Bit-Wert (2 bytes) als Dezimalzahl ausgeben
;  ReadChar        - Ein Zeichen einlesen
;  ReadDec         - Eine Dezimalzahl (< 2^16) einlesen

;********************************************************************************************************

LF      EQU     0AH
CR      EQU     0DH
        
;********************************************************************************************************

        BSEG
        
Zf:     DBIT    1

        CSEG
                
;********************************************************************************************************
; Baudrateneinstellungen:
; SMOD = 1 ==> TH1 = 256 - (2 * f/Hz) / (384 * BR)

; 22,1184 MHz -   2400 : 208     24 MHz - 2400 : 204        12 MHz - 2400 : 230
;             -   4800 : 232            - 4800 : 230               - 4800 : 243
;             -   9600 : 244            - 9600 : 243
;             -  19200 : 250
;             -  38400 : 253
;             -  57600 : 254
;             - 115200 : 255

;********************************************************************************************************
; RS232 initalisieren

InitDefSer:
        MOV     A,#244          ; BR-Count (s.o.) - 9600 baud
InitSerial:   
        MOV     TH1,A           ; load value

        MOV     A,TMOD
        ANL     A,#0FH
        ADD     A,#20H
        MOV     TMOD,A          ; timer 1 mode 2 (8 bit, auto-reload)
        CLR     SM0             ; serial mode 1 (8-bit)
        SETB    SM1
        SETB    TR1             ; start timer
        MOV     A,PCON
        ORL     A,#80H
        MOV     PCON,A          ; SMOD=1 (Baudrate x 2)
        SETB    REN             ; Enable receiver
        SETB    TI              ; set TI for transmit buffer empty
        CLR     RI              ; clear SI for receive interrupt
        RET

;********************************************************************************************************
; Ein Zeichen ber interne ser. Schnittstelle ausgeben
; Zeichen in A

WriteChar:
WWAIT:  JNB     TI,WWAIT        ; warten
        MOV     SBUF,A          ; Ausgabe
        CLR     TI
WRET:   RET

         
;********************************************************************************************************
; Nullterm. String in Daten-Speicher ber interne ser. Schnittstelle ausgeben 
; und Cr-Lf anhngen
; Zeiger auf String in DPTR

WriteString:
        MOVX    A,@DPTR
        INC     DPTR
        JZ      WriteCrLf
        ACALL   WRITECHAR
        SJMP    WRITESTRING
        
;********************************************************************************************************
; Cr+Lf (neue Zeile) ber interne ser. Schnittstelle ausgeben 

WriteCrLf:
        MOV     A,#CR
        ACALL   WRITECHAR
        MOV     A,#LF
        ACALL   WRITECHAR
        RET

;********************************************************************************************************
; Nulltem. String aus Code-Mem. ber interne ser. Schnittstelle ausgeben 
; Zeiger auf String in DPTR

WriteFixStr:
        CLR     A 
        MOVC    A,@A+DPTR
        INC     DPTR
        JZ      WFEND
        ACALL   WRITECHAR
        SJMP    WRITEFIXSTR
        
WFEND:  RET

;********************************************************************************************************
; Byte als Dezimalzahl ausgeben
; Wert in A
; Carry = 0 : mit fhrenden Nullen
;       = 1 : ohne fhrende Nullen
; Verwendet ZF,B

WriteDecByte:
        MOV     ZF,C            ; speicher Flag
        MOV     B,#100
        DIV     AB              ; 1. Stelle in A, Rest in B
        JZ      WD3
        CLR     ZF
        JMP     WD2             ; nicht Null
        
WD3:    JNB     ZF,WD2          ; mit fhrender 0
        JZ      WD1
WD2:    ADD     A,#30H          ; ASCII-Code
        CALL    WRITECHAR
WD1:    MOV     A,B
        MOV     B,#10
        DIV     AB              ; 2. Stelle
        JZ      WD6
        CLR     ZF
        JMP     WD5             ; nicht Null

WD6:    JNB     ZF,WD5          ; mit fhrender 0
        JZ      WD4
WD5:    ADD     A,#30H          ; ASCII-Code
        CALL    WRITECHAR
WD4:    MOV     A,B             ; 3. Stelle
        ADD     A,#30H          ; ASCII-Code
        CALL    WRITECHAR
        RET

;********************************************************************************************************
; 16-bit-Zahl als Dezimalzahl ausgeben
; Wert in A,B  (low,high)
; Carry = 0 : mit fhrenden Nullen
;       = 1 : ohne fhrende Nullen
; Verwendet ZF,R2,R3,R4,R5,R6,R7

WriteDec:
        MOV     ZF,C            ; speicher Flag
        MOV     R2,A            ; Werte (low)
        MOV     R3,B            ; Wert (high)
        MOV     R7,#3           
WD21:   MOV     A,R7
        RL      A
        MOV     B,A
        ADD     A,#DivTab-WD2L-3
WD2L:   MOVC    A,@A+PC
        MOV     R4,A            ; Divisor (low)
        MOV     A,B
        ADD     A,#DivTab-WD2H-2
WD2H:   MOVC    A,@A+PC
        MOV     R5,A            ; Divisor (high)
        
        MOV     R6,#0           ; Quotient
        CLR     C     
WD22:   MOV     A,R2
        SUBB    A,R4
        MOV     B,A             ; zwischenspeichern
        MOV     A,R3
        SUBB    A,R5
        JC      WD23
        MOV     R3,A            ; speichern
        MOV     R2,B
        INC     R6
        JMP     WD22
          
WD23:   MOV     A,R6            ; Quotient  
        JNB     ZF,WD24         ; mit fhrender 0
        JZ      WD25
WD24:   CLR     ZF
        ADD     A,#30H          ; nach ASCII   
        CALL    WriteChar
WD25:   DJNZ    R7,WD21         ; nchste Stelle
        
        MOV     A,R2
        MOV     B,#10           ; Zehner
        DIV     AB
        JNB     ZF,WD26         ; mit fhrender 0
        JZ      WD27
WD26:   CLR     ZF
        ADD     A,#30H          ; nach ASCII   
        CALL    WriteChar
WD27:   MOV     A,B             ; Einer
        ADD     A,#30H          ; nach ASCII   
        CALL    WriteChar
        RET

DivTab:  DB     100,0,1000 mod 256,1000/256,10000 mod 256,10000/256
               
;********************************************************************************************************
; Warten bis ein Zeichen von der seriellen Schnittstelle empfangen wurde
; Empfangenes Zeichen in A

ReadChar:
        JNB     RI,READCHAR     ; warten
        CLR     RI
        MOV     A,SBUF          ; Zeichen lesen
        RET

;********************************************************************************************************
; Warten bis eine Zahl (< 2^16) von der seriellen Schnittstelle empfangen wurde
; Empfangene Zahl in A,B  (low,high)
; Verwendet R5,R6,R7

ReadDec:
        MOV     R7,#0           ; Ergebnis inital.
        MOV     R6,#0
RD0:    CALL    READCHAR
        CALL    WRITECHAR       ; Echo
RD1:    CLR     C
        SUBB    A,#30H          ; - 48
        JC      RDRET           ; < 48 ('0')
        SUBB    A,#0AH          ; - 10
        JNC     RDRET           ; > 9  ('9')
        ADD     A,#0AH          ; Wert 0 .. 9
        MOV     R5,A            ; zwischenspeichern
        
        MOV     A,R6            ; Ergebnis holen - low
        MOV     B,#10D
        MUL     AB              ; * 10
        ADD     A,R5            ; addiere nchste Stelle
        JNC     RD2
        INC     B               ; bertrag
RD2:    MOV     R6,A            ; Ergebnis speichern - low
        MOV     R5,B            ; bertrag
        MOV     A,R7            ; Ergebnis holen - high
        MOV     B,#10D
        MUL     AB              ; * 10
        ADD     A,R5            ; bertrag addieren
        MOV     R7,A            ; Ergebnis speichern - high
        SJMP    RD0            

RDRET:  MOV     A,R6
        MOV     B,R7
        RET    

$LIST
